@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
+ * Copyright (c) 2000, 2015 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -33,7 +33,7 @@ public abstract class Device implements Drawable {
 	 * within the packages provided by SWT. It is not available on all
 	 * platforms and should never be accessed from application code.
 	 * </p>
-	 * 
+	 *
 	 * @noreference This field is not intended to be referenced by clients.
 	 */
 	protected int /*long*/ xDisplay;
@@ -46,14 +46,14 @@ public abstract class Device implements Drawable {
 	Error [] errors;
 	Object [] objects;
 	Object trackingLock;
-	
+
 	/* Colormap and reference count */
 	GdkColor [] gdkColors;
 	int [] colorRefCount;
-	
+
 	/* Disposed flag */
 	boolean disposed;
-	
+
 	/* Warning and Error Handlers */
 	int /*long*/ logProc;
 	Callback logCallback;
@@ -61,7 +61,7 @@ public abstract class Device implements Drawable {
 	String [] log_domains = {"", "GLib-GObject", "GLib", "GObject", "Pango", "ATK", "GdkPixbuf", "Gdk", "Gtk", "GnomeVFS", "GIO"};
 	int [] handler_ids = new int [log_domains.length];
 	int warningLevel;
-	
+
 	/* X Warning and Error Handlers */
 	static Callback XErrorCallback, XIOErrorCallback;
 	static int /*long*/ XErrorProc, XIOErrorProc, XNullErrorProc, XNullIOErrorProc;
@@ -73,15 +73,15 @@ public abstract class Device implements Drawable {
 	* palette.
 	*/
 	Color COLOR_BLACK, COLOR_DARK_RED, COLOR_DARK_GREEN, COLOR_DARK_YELLOW, COLOR_DARK_BLUE;
-	Color COLOR_DARK_MAGENTA, COLOR_DARK_CYAN, COLOR_GRAY, COLOR_DARK_GRAY, COLOR_RED;
+	Color COLOR_DARK_MAGENTA, COLOR_DARK_CYAN, COLOR_GRAY, COLOR_DARK_GRAY, COLOR_RED, COLOR_TRANSPARENT;
 	Color COLOR_GREEN, COLOR_YELLOW, COLOR_BLUE, COLOR_MAGENTA, COLOR_CYAN, COLOR_WHITE;
 
 	/* System Font */
 	Font systemFont;
-	
+
 	/* Device dpi */
 	Point dpi;
-	
+
 	int /*long*/ emptyTab;
 
 	boolean useXRender;
@@ -103,7 +103,7 @@ public abstract class Device implements Drawable {
 		try {
 			Class.forName ("org.eclipse.swt.widgets.Display");
 		} catch (ClassNotFoundException e) {}
-	}	
+	}
 
 /*
 * TEMPORARY CODE.
@@ -118,12 +118,12 @@ static synchronized Device getDevice () {
 /**
  * Constructs a new instance of this class.
  * <p>
- * You must dispose the device when it is no longer required. 
+ * You must dispose the device when it is no longer required.
  * </p>
  *
  * @see #create
  * @see #init
- * 
+ *
  * @since 3.1
  */
 public Device() {
@@ -133,7 +133,7 @@ public Device() {
 /**
  * Constructs a new instance of this class.
  * <p>
- * You must dispose the device when it is no longer required. 
+ * You must dispose the device when it is no longer required.
  * </p>
  *
  * @param data the DeviceData which describes the receiver
@@ -364,7 +364,7 @@ public DeviceData getDeviceData () {
 /**
  * Returns a rectangle which describes the area of the
  * receiver which is capable of displaying data.
- * 
+ *
  * @return the client area
  *
  * @exception SWTException <ul>
@@ -381,7 +381,7 @@ public Rectangle getClientArea () {
 /**
  * Returns the bit depth of the screen, which is the number of
  * bits it takes to represent the number of unique colors that
- * the screen is currently capable of displaying. This number 
+ * the screen is currently capable of displaying. This number
  * will typically be one of 1, 8, 15, 16, 24 or 32.
  *
  * @return the depth of the screen
@@ -503,6 +503,7 @@ Point getScreenDPI () {
 public Color getSystemColor (int id) {
 	checkDevice ();
 	switch (id) {
+		case SWT.COLOR_TRANSPARENT:			return COLOR_TRANSPARENT;
 		case SWT.COLOR_BLACK: 				return COLOR_BLACK;
 		case SWT.COLOR_DARK_RED: 			return COLOR_DARK_RED;
 		case SWT.COLOR_DARK_GREEN:	 		return COLOR_DARK_GREEN;
@@ -573,12 +574,12 @@ public boolean getWarnings () {
  * If subclasses reimplement this method, they must
  * call the <code>super</code> implementation.
  * </p>
- * 
+ *
  * @see #create
  */
 protected void init () {
 	this.dpi = getDPI();
-	
+
 	if (xDisplay != 0 && !OS.USE_CAIRO) {
 		int[] event_basep = new int[1], error_basep = new int [1];
 		if (OS.XRenderQueryExtension (xDisplay, event_basep, error_basep)) {
@@ -590,11 +591,11 @@ protected void init () {
 
 	//TODO: Remove; temporary code only
 	boolean fixAIX = OS.IsAIX && OS.PTR_SIZEOF == 8;
-	
+
 	if (debug || fixAIX) {
 		if (xDisplay != 0) {
 			/* Create the warning and error callbacks */
-			Class clazz = getClass ();
+			Class<?> clazz = getClass ();
 			synchronized (clazz) {
 				int index = 0;
 				while (index < Devices.length) {
@@ -615,13 +616,13 @@ protected void init () {
 			if (debug) OS.XSynchronize (xDisplay, true);
 		}
 	}
-	
+
 	/* Create GTK warnings and error callbacks */
 	if (xDisplay != 0) {
 		logCallback = new Callback (this, "logProc", 4);
 		logProc = logCallback.getAddress ();
 		if (logProc == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
-		
+
 		/* Set GTK warning and error handlers */
 		if (debug) {
 			int flags = OS.G_LOG_LEVEL_MASK | OS.G_LOG_FLAG_FATAL | OS.G_LOG_FLAG_RECURSION;
@@ -631,8 +632,9 @@ protected void init () {
 			}
 		}
 	}
-	
+
 	/* Create the standard colors */
+	COLOR_TRANSPARENT = new Color (this, 0xFF,0xFF,0xFF,0);
 	COLOR_BLACK = new Color (this, 0,0,0);
 	COLOR_DARK_RED = new Color (this, 0x80,0,0);
 	COLOR_DARK_GREEN = new Color (this, 0,0x80,0);
@@ -661,12 +663,12 @@ protected void init () {
 	/* Initialize the system font slot */
 	int /*long*/ defaultFont;
 	if (OS.GTK3) {
-		int /*long*/ context = OS.gtk_widget_get_style_context (shellHandle);	
+		int /*long*/ context = OS.gtk_widget_get_style_context (shellHandle);
 		defaultFont = OS.gtk_style_context_get_font (context, OS.GTK_STATE_FLAG_NORMAL);
 	} else {
-		int /*long*/ style = OS.gtk_widget_get_style (shellHandle);	
+		int /*long*/ style = OS.gtk_widget_get_style (shellHandle);
 		defaultFont = OS.gtk_style_get_font_desc (style);
-	}	
+	}
 	defaultFont = OS.pango_font_description_copy (defaultFont);
 	Point dpi = getDPI(), screenDPI = getScreenDPI();
 	if (dpi.y != screenDPI.y) {
@@ -676,7 +678,7 @@ protected void init () {
 	systemFont = Font.gtk_new (this, defaultFont);
 }
 
-/**	 
+/**
  * Invokes platform specific functionality to allocate a new GC handle.
  * <p>
  * <b>IMPORTANT:</b> This method is <em>not</em> part of the public
@@ -686,14 +688,14 @@ protected void init () {
  * application code.
  * </p>
  *
- * @param data the platform specific GC data 
+ * @param data the platform specific GC data
  * @return the platform specific GC handle
- * 
+ *
  * @noreference This method is not intended to be referenced by clients.
  */
 public abstract int /*long*/ internal_new_GC (GCData data);
 
-/**	 
+/**
  * Invokes platform specific functionality to dispose a GC handle.
  * <p>
  * <b>IMPORTANT:</b> This method is <em>not</em> part of the public
@@ -704,8 +706,8 @@ public abstract int /*long*/ internal_new_GC (GCData data);
  * </p>
  *
  * @param hDC the platform specific GC handle
- * @param data the platform specific GC data 
- * 
+ * @param data the platform specific GC data
+ *
  * @noreference This method is not intended to be referenced by clients.
  */
 public abstract void internal_dispose_GC (int /*long*/ hDC, GCData data);
@@ -739,7 +741,7 @@ public boolean isDisposed () {
  * </ul>
  *
  * @see Font
- * 
+ *
  * @since 3.3
  */
 public boolean loadFont (String path) {
@@ -821,7 +823,7 @@ static synchronized void register (Device device) {
 protected void release () {
 	if (shellHandle != 0) OS.gtk_widget_destroy(shellHandle);
 	shellHandle = 0;
-	
+
 	/* Dispose the default font */
 	if (systemFont != null) systemFont.dispose ();
 	systemFont = null;
@@ -862,7 +864,7 @@ protected void release () {
 	COLOR_BLACK = COLOR_DARK_RED = COLOR_DARK_GREEN = COLOR_DARK_YELLOW = COLOR_DARK_BLUE =
 	COLOR_DARK_MAGENTA = COLOR_DARK_CYAN = COLOR_GRAY = COLOR_DARK_GRAY = COLOR_RED =
 	COLOR_GREEN = COLOR_YELLOW = COLOR_BLUE = COLOR_MAGENTA = COLOR_CYAN = COLOR_WHITE = null;
-		
+
 	if (emptyTab != 0) OS.pango_tab_array_free(emptyTab);
 	emptyTab = 0;
 
@@ -953,4 +955,20 @@ static int /*long*/ XIOErrorProc (int /*long*/ xDisplay) {
 	return 0;
 }
 
+/**
+ * Returns DPI in x direction. In the modern monitors DPI for
+ * X and Y directions is same.
+ *
+ * @return the horizontal DPI
+ */
+int _getDPIx () {
+	int /*long*/ screen = OS.gdk_screen_get_default();
+	int monitor = OS.gdk_screen_get_monitor_at_point(screen, 0, 0);
+
+	GdkRectangle dest = new GdkRectangle ();
+	OS.gdk_screen_get_monitor_geometry(screen, monitor, dest);
+	int widthMM = OS.gdk_screen_get_monitor_width_mm(screen, monitor);
+	return Compatibility.round (254 * dest.width, widthMM * 10);
+}
+
 }
