@@ -60,6 +60,9 @@ public class Path extends Resource {
  * graphics subsystem which may not be available on some
  * platforms.
  * </p>
+ * <p>
+ * You must dispose the path when it is no longer required.
+ * </p>
  * 
  * @param device the device on which to allocate the path
  * 
@@ -97,6 +100,9 @@ public Path (Device device) {
  * graphics subsystem which may not be available on some
  * platforms.
  * </p>
+ * <p>
+ * You must dispose the path when it is no longer required.
+ * </p>
  * 
  * @param device the device on which to allocate the path
  * @param path the path to make a copy
@@ -146,12 +152,15 @@ public Path (Device device, Path path, float flatness) {
 }
 
 /**
- * Constructs a new Path with the specifed PathData.
+ * Constructs a new Path with the specified PathData.
  * <p>
  * This operation requires the operating system's advanced
  * graphics subsystem which may not be available on some
  * platforms.
  * </p>
+ * <p>
+ * You must dispose the path when it is no longer required.
+ * </p>
  * 
  * @param device the device on which to allocate the path
  * @param data the data for the path
@@ -642,6 +651,7 @@ public void quadTo(float cx, float cy, float x, float y) {
 	closed = false;
 }
 
+@Override
 void destroy() {
 	Cairo.cairo_destroy(handle);
 	handle = 0;
@@ -684,6 +694,7 @@ void init(PathData data) {
  *
  * @return <code>true</code> when the Path is disposed, and <code>false</code> otherwise
  */
+@Override
 public boolean isDisposed() {
 	return handle == 0;
 }
@@ -694,6 +705,7 @@ public boolean isDisposed() {
  *
  * @return a string representation of the receiver
  */
+@Override
 public String toString() {
 	if (isDisposed()) return "Path {*DISPOSED*}";
 	return "Path {" + handle + "}";
