@@ -90,10 +90,8 @@ public class Cairo extends Platform {
 	public static final int CAIRO_PATH_CLOSE_PATH = 3;
 	
 /** 64*/
-public static final native int cairo_font_extents_t_sizeof ();
 public static final native int cairo_path_data_t_sizeof ();
 public static final native int cairo_path_t_sizeof ();
-public static final native int cairo_text_extents_t_sizeof ();
 	
 /** Natives */
 public static final native int CAIRO_VERSION_ENCODE(int major, int minor, int micro);
@@ -141,16 +139,6 @@ public static final void cairo_clip(int /*long*/ cr) {
 	}
 }
 /** @param cr cast=(cairo_t *) */
-public static final native void _cairo_clip_preserve(int /*long*/ cr);
-public static final void cairo_clip_preserve(int /*long*/ cr) {
-	lock.lock();
-	try {
-		_cairo_clip_preserve(cr);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
 public static final native void _cairo_close_path(int /*long*/ cr);
 public static final void cairo_close_path(int /*long*/ cr) {
 	lock.lock();
@@ -221,26 +209,6 @@ public static final void cairo_destroy(int /*long*/ cr) {
 	}
 }
 /** @param cr cast=(cairo_t *) */
-public static final native void _cairo_device_to_user(int /*long*/ cr, double[] x, double[] y);
-public static final void cairo_device_to_user(int /*long*/ cr, double[] x, double[] y) {
-	lock.lock();
-	try {
-		_cairo_device_to_user(cr, x, y);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
-public static final native void _cairo_device_to_user_distance(int /*long*/ cr, double[] dx, double[] dy);
-public static final void cairo_device_to_user_distance(int /*long*/ cr, double[] dx, double[] dy) {
-	lock.lock();
-	try {
-		_cairo_device_to_user_distance(cr, dx, dy);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
 public static final native void _cairo_fill(int /*long*/ cr);
 public static final void cairo_fill(int /*long*/ cr) {
 	lock.lock();
@@ -250,36 +218,6 @@ public static final void cairo_fill(int /*long*/ cr) {
 		lock.unlock();
 	}
 }
-/** @param cr cast=(cairo_t *) */
-public static final native void _cairo_fill_extents(int /*long*/ cr, double[] x1, double[] y1, double[] x2, double[] y2);
-public static final void cairo_fill_extents(int /*long*/ cr, double[] x1, double[] y1, double[] x2, double[] y2) {
-	lock.lock();
-	try {
-		_cairo_fill_extents(cr, x1, y1, x2, y2);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
-public static final native void _cairo_fill_preserve(int /*long*/ cr);
-public static final void cairo_fill_preserve(int /*long*/ cr) {
-	lock.lock();
-	try {
-		_cairo_fill_preserve(cr);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
-public static final native void _cairo_font_extents(int /*long*/ cr, cairo_font_extents_t extents);
-public static final void cairo_font_extents(int /*long*/ cr, cairo_font_extents_t extents) {
-	lock.lock();
-	try {
-		_cairo_font_extents(cr, extents);
-	} finally {
-		lock.unlock();
-	}
-}
 public static final native int /*long*/ _cairo_font_options_create();
 public static final int /*long*/ cairo_font_options_create() {
 	lock.lock();
@@ -319,16 +257,6 @@ public static final void cairo_font_options_set_antialias(int /*long*/ options,
 		lock.unlock();
 	}
 }
-/** @method flags=dynamic */
-public static final native int _cairo_format_stride_for_width(int format, int width);
-public static final int cairo_format_stride_for_width(int format, int width) {
-	lock.lock();
-	try {
-		return _cairo_format_stride_for_width(format, width);
-	} finally {
-		lock.unlock();
-	}
-}
 /** @param cr cast=(cairo_t *) */
 public static final native int _cairo_get_antialias(int /*long*/ cr);
 public static final int cairo_get_antialias(int /*long*/ cr) {
@@ -369,62 +297,6 @@ public static final int /*long*/ cairo_get_font_face(int /*long*/ cr) {
 		lock.unlock();
 	}
 }
-/**
- * @param cr cast=(cairo_t *)
- * @param matrix cast=(cairo_matrix_t *)
- */
-public static final native void _cairo_get_font_matrix(int /*long*/ cr, double[] matrix);
-public static final void cairo_get_font_matrix(int /*long*/ cr, double[] matrix) {
-	lock.lock();
-	try {
-		_cairo_get_font_matrix(cr, matrix);
-	} finally {
-		lock.unlock();
-	}
-}
-/**
- * @param cr cast=(cairo_t *)
- * @param options cast=(cairo_font_options_t *)
- */
-public static final native void _cairo_get_font_options(int /*long*/ cr, int /*long*/ options);
-public static final void cairo_get_font_options(int /*long*/ cr, int /*long*/ options) {
-	lock.lock();
-	try {
-		_cairo_get_font_options(cr, options);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
-public static final native int _cairo_get_line_cap(int /*long*/ cr);
-public static final int cairo_get_line_cap(int /*long*/ cr) {
-	lock.lock();
-	try {
-		return _cairo_get_line_cap(cr);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
-public static final native int _cairo_get_line_join(int /*long*/ cr);
-public static final int cairo_get_line_join(int /*long*/ cr) {
-	lock.lock();
-	try {
-		return _cairo_get_line_join(cr);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
-public static final native double _cairo_get_line_width(int /*long*/ cr);
-public static final double cairo_get_line_width(int /*long*/ cr) {
-	lock.lock();
-	try {
-		return _cairo_get_line_width(cr);
-	} finally {
-		lock.unlock();
-	}
-}
 /**
  * @param cr cast=(cairo_t *)
  * @param matrix cast=(cairo_matrix_t *)
@@ -439,16 +311,6 @@ public static final void cairo_get_matrix(int /*long*/ cr, double[] matrix) {
 	}
 }
 /** @param cr cast=(cairo_t *) */
-public static final native double _cairo_get_miter_limit(int /*long*/ cr);
-public static final double cairo_get_miter_limit(int /*long*/ cr) {
-	lock.lock();
-	try {
-		return _cairo_get_miter_limit(cr);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
 public static final native int _cairo_get_operator(int /*long*/ cr);
 public static final int cairo_get_operator(int /*long*/ cr) {
 	lock.lock();
@@ -488,33 +350,6 @@ public static final double cairo_get_tolerance(int /*long*/ cr) {
 		lock.unlock();
 	}
 }
-/**
- * @param cr cast=(cairo_t *)
- * @param glyphs cast=(cairo_glyph_t *)
- * @param extents cast=(cairo_text_extents_t *)
- */
-public static final native void _cairo_glyph_extents(int /*long*/ cr, int /*long*/ glyphs, int num_glyphs, int /*long*/ extents);
-public static final void cairo_glyph_extents(int /*long*/ cr, int /*long*/ glyphs, int num_glyphs, int /*long*/ extents) {
-	lock.lock();
-	try {
-		_cairo_glyph_extents(cr, glyphs, num_glyphs, extents);
-	} finally {
-		lock.unlock();
-	}
-}
-/**
- * @param cr cast=(cairo_t *)
- * @param glyphs cast=(cairo_glyph_t *)
- */
-public static final native void _cairo_glyph_path(int /*long*/ cr, int /*long*/ glyphs, int num_glyphs);
-public static final void cairo_glyph_path(int /*long*/ cr, int /*long*/ glyphs, int num_glyphs) {
-	lock.lock();
-	try {
-		_cairo_glyph_path(cr, glyphs, num_glyphs);
-	} finally {
-		lock.unlock();
-	}
-}
 /** @param cr cast=(cairo_t *) */
 public static final native void _cairo_identity_matrix(int /*long*/ cr);
 public static final void cairo_identity_matrix(int /*long*/ cr) {
@@ -534,16 +369,6 @@ public static final int /*long*/ cairo_image_surface_create(int format, int widt
 		lock.unlock();
 	}
 }
-/** @param data cast=(unsigned char *) */
-public static final native int /*long*/ _cairo_image_surface_create_for_data(int /*long*/ data, int format, int width, int height, int stride);
-public static final int /*long*/ cairo_image_surface_create_for_data(int /*long*/ data, int format, int width, int height, int stride) {
-	lock.lock();
-	try {
-		return _cairo_image_surface_create_for_data(data, format, width, height, stride);
-	} finally {
-		lock.unlock();
-	}
-}
 /**
  * @method flags=dynamic
  * @param surface cast=(cairo_surface_t *)
@@ -834,16 +659,6 @@ public static final void cairo_path_destroy(int /*long*/ path) {
 	}
 }
 /** @param pattern cast=(cairo_pattern_t *) */
-public static final native void _cairo_pattern_add_color_stop_rgb(int /*long*/ pattern, double offset, double red, double green, double blue);
-public static final void cairo_pattern_add_color_stop_rgb(int /*long*/ pattern, double offset, double red, double green, double blue) {
-	lock.lock();
-	try {
-		_cairo_pattern_add_color_stop_rgb(pattern, offset, red, green, blue);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param pattern cast=(cairo_pattern_t *) */
 public static final native void _cairo_pattern_add_color_stop_rgba(int /*long*/ pattern, double offset, double red, double green, double blue, double alpha);
 public static final void cairo_pattern_add_color_stop_rgba(int /*long*/ pattern, double offset, double red, double green, double blue, double alpha) {
 	lock.lock();
@@ -872,15 +687,6 @@ public static final int /*long*/ cairo_pattern_create_linear(double x0, double y
 		lock.unlock();
 	}
 }
-public static final native int /*long*/ _cairo_pattern_create_radial(double cx0, double cy0, double radius0, double cx1, double cy1, double radius1);
-public static final int /*long*/ cairo_pattern_create_radial(double cx0, double cy0, double radius0, double cx1, double cy1, double radius1) {
-	lock.lock();
-	try {
-		return _cairo_pattern_create_radial(cx0, cy0, radius0, cx1, cy1, radius1);
-	} finally {
-		lock.unlock();
-	}
-}
 /** @param pattern cast=(cairo_pattern_t *) */
 public static final native void _cairo_pattern_destroy(int /*long*/ pattern);
 public static final void cairo_pattern_destroy(int /*long*/ pattern) {
@@ -902,39 +708,6 @@ public static final int cairo_pattern_get_extend(int /*long*/ pattern) {
 	}
 }
 /** @param pattern cast=(cairo_pattern_t *) */
-public static final native int _cairo_pattern_get_filter(int /*long*/ pattern);
-public static final int cairo_pattern_get_filter(int /*long*/ pattern) {
-	lock.lock();
-	try {
-		return _cairo_pattern_get_filter(pattern);
-	} finally {
-		lock.unlock();
-	}
-}
-/**
- * @param pattern cast=(cairo_pattern_t *)
- * @param matrix cast=(cairo_matrix_t *)
- */
-public static final native void _cairo_pattern_get_matrix(int /*long*/ pattern, double[] matrix);
-public static final void cairo_pattern_get_matrix(int /*long*/ pattern, double[] matrix) {
-	lock.lock();
-	try {
-		_cairo_pattern_get_matrix(pattern, matrix);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param pattern cast=(cairo_pattern_t *) */
-public static final native void _cairo_pattern_reference(int /*long*/ pattern);
-public static final void cairo_pattern_reference(int /*long*/ pattern) {
-	lock.lock();
-	try {
-		_cairo_pattern_reference(pattern);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param pattern cast=(cairo_pattern_t *) */
 public static final native void _cairo_pattern_set_extend(int /*long*/ pattern, int extend);
 public static final void cairo_pattern_set_extend(int /*long*/ pattern, int extend) {
 	lock.lock();
@@ -1040,36 +813,6 @@ public static final int /*long*/ cairo_reference(int /*long*/ cr) {
 	}
 }
 /** @param cr cast=(cairo_t *) */
-public static final native void _cairo_rel_curve_to(int /*long*/ cr, double dx1, double dy1, double dx2, double dy2, double dx3, double dy3);
-public static final void cairo_rel_curve_to(int /*long*/ cr, double dx1, double dy1, double dx2, double dy2, double dx3, double dy3) {
-	lock.lock();
-	try {
-		_cairo_rel_curve_to(cr, dx1, dy1, dx2, dy2, dx3, dy3);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
-public static final native void _cairo_rel_line_to(int /*long*/ cr, double dx, double dy);
-public static final void cairo_rel_line_to(int /*long*/ cr, double dx, double dy) {
-	lock.lock();
-	try {
-		_cairo_rel_line_to(cr, dx, dy);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
-public static final native void _cairo_rel_move_to(int /*long*/ cr, double dx, double dy);
-public static final void cairo_rel_move_to(int /*long*/ cr, double dx, double dy) {
-	lock.lock();
-	try {
-		_cairo_rel_move_to(cr, dx, dy);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
 public static final native void _cairo_reset_clip(int /*long*/ cr);
 public static final void cairo_reset_clip(int /*long*/ cr) {
 	lock.lock();
@@ -1175,32 +918,6 @@ public static final void cairo_set_font_face(int /*long*/ cr, int /*long*/ font_
 		lock.unlock();
 	}
 }
-/**
- * @param cr cast=(cairo_t *)
- * @param matrix cast=(cairo_matrix_t *)
- */
-public static final native void _cairo_set_font_matrix(int /*long*/ cr, double[] matrix);
-public static final void cairo_set_font_matrix(int /*long*/ cr, double[] matrix) {
-	lock.lock();
-	try {
-		_cairo_set_font_matrix(cr, matrix);
-	} finally {
-		lock.unlock();
-	}
-}
-/**
- * @param cr cast=(cairo_t *)
- * @param options cast=(cairo_font_options_t *)
- */
-public static final native void _cairo_set_font_options(int /*long*/ cr, int /*long*/ options);
-public static final void cairo_set_font_options(int /*long*/ cr, int /*long*/ options) {
-	lock.lock();
-	try {
-		_cairo_set_font_options(cr, options);
-	} finally {
-		lock.unlock();
-	}
-}
 /** @param cr cast=(cairo_t *) */
 public static final native void _cairo_set_font_size(int /*long*/ cr, double size);
 public static final void cairo_set_font_size(int /*long*/ cr, double size) {
@@ -1330,19 +1047,6 @@ public static final void cairo_set_tolerance(int /*long*/ cr, double tolerance)
 		lock.unlock();
 	}
 }
-/**
- * @param cr cast=(cairo_t *)
- * @param glyphs cast=(cairo_glyph_t *)
- */
-public static final native void _cairo_show_glyphs(int /*long*/ cr, int /*long*/ glyphs, int num_glyphs);
-public static final void cairo_show_glyphs(int /*long*/ cr, int /*long*/ glyphs, int num_glyphs) {
-	lock.lock();
-	try {
-		_cairo_show_glyphs(cr, glyphs, num_glyphs);
-	} finally {
-		lock.unlock();
-	}
-}
 /** @param cr cast=(cairo_t *) */
 public static final native void _cairo_show_page(int /*long*/ cr);
 public static final void cairo_show_page(int /*long*/ cr) {
@@ -1353,38 +1057,6 @@ public static final void cairo_show_page(int /*long*/ cr) {
 		lock.unlock();
 	}
 }
-/**
- * @param cr cast=(cairo_t *)
- * @param utf8 cast=(const char *)
- */
-public static final native void _cairo_show_text(int /*long*/ cr, byte[] utf8);
-public static final void cairo_show_text(int /*long*/ cr, byte[] utf8) {
-	lock.lock();
-	try {
-		_cairo_show_text(cr, utf8);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
-public static final native int _cairo_status(int /*long*/ cr);
-public static final int cairo_status(int /*long*/ cr) {
-	lock.lock();
-	try {
-		return _cairo_status(cr);
-	} finally {
-		lock.unlock();
-	}
-}
-public static final native int /*long*/ _cairo_status_to_string(int status);
-public static final int /*long*/ cairo_status_to_string(int status) {
-	lock.lock();
-	try {
-		return _cairo_status_to_string(status);
-	} finally {
-		lock.unlock();
-	}
-}
 /** @param cr cast=(cairo_t *) */
 public static final native void _cairo_stroke(int /*long*/ cr);
 public static final void cairo_stroke(int /*long*/ cr) {
@@ -1395,26 +1067,6 @@ public static final void cairo_stroke(int /*long*/ cr) {
 		lock.unlock();
 	}
 }
-/** @param cr cast=(cairo_t *) */
-public static final native void _cairo_stroke_extents(int /*long*/ cr, double[] x1, double[] y1, double[] x2, double[] y2);
-public static final void cairo_stroke_extents(int /*long*/ cr, double[] x1, double[] y1, double[] x2, double[] y2) {
-	lock.lock();
-	try {
-		_cairo_stroke_extents(cr, x1, y1, x2, y2);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
-public static final native void _cairo_stroke_preserve(int /*long*/ cr);
-public static final void cairo_stroke_preserve(int /*long*/ cr) {
-	lock.lock();
-	try {
-		_cairo_stroke_preserve(cr);
-	} finally {
-		lock.unlock();
-	}
-}
 /** @param other cast=(cairo_surface_t *) */
 public static final native int /*long*/ _cairo_surface_create_similar(int /*long*/ other, int format, int width, int height);
 public static final int /*long*/ cairo_surface_create_similar(int /*long*/ other, int format, int width, int height) {
@@ -1514,68 +1166,6 @@ public static final void cairo_surface_reference(int /*long*/ surface) {
 		lock.unlock();
 	}
 }
-/** @param surface cast=(cairo_surface_t *) */
-public static final native void _cairo_surface_set_device_offset(int /*long*/ surface, double x_offset, double y_offset);
-public static final void cairo_surface_set_device_offset(int /*long*/ surface, double x_offset, double y_offset) {
-	lock.lock();
-	try {
-		_cairo_surface_set_device_offset(surface, x_offset, y_offset);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @method flags=dynamic */
-public static final native void _cairo_surface_set_fallback_resolution(int /*long*/ surface, double x_pixels_per_inch, double y_pixels_per_inch);
-public static final void cairo_surface_set_fallback_resolution(int /*long*/ surface, double x_pixels_per_inch, double y_pixels_per_inch) {
-	lock.lock();
-	try {
-		_cairo_surface_set_fallback_resolution(surface, x_pixels_per_inch, y_pixels_per_inch);
-	} finally {
-		lock.unlock();
-	}
-}
-/**
- * @param surface cast=(cairo_surface_t *)
- * @param key cast=(cairo_user_data_key_t *)
- * @param user_data cast=(void *)
- * @param destroy cast=(cairo_destroy_func_t)
- */
-public static final native int _cairo_surface_set_user_data(int /*long*/ surface, int /*long*/ key, int /*long*/ user_data, int /*long*/ destroy);
-public static final int cairo_surface_set_user_data(int /*long*/ surface, int /*long*/ key, int /*long*/ user_data, int /*long*/ destroy) {
-	lock.lock();
-	try {
-		return _cairo_surface_set_user_data(surface, key, user_data, destroy);
-	} finally {
-		lock.unlock();
-	}
-}
-/**
- * @param cr cast=(cairo_t *)
- * @param utf8 cast=(const char *)
- * @param extents cast=(cairo_text_extents_t *)
- */
-public static final native void _cairo_text_extents(int /*long*/ cr, byte[] utf8, cairo_text_extents_t extents);
-public static final void cairo_text_extents(int /*long*/ cr, byte[] utf8, cairo_text_extents_t extents) {
-	lock.lock();
-	try {
-		_cairo_text_extents(cr, utf8, extents);
-	} finally {
-		lock.unlock();
-	}
-}
-/**
- * @param cr cast=(cairo_t *)
- * @param utf8 cast=(const char *)
- */
-public static final native void _cairo_text_path(int /*long*/ cr, byte[] utf8);
-public static final void cairo_text_path(int /*long*/ cr, byte[] utf8) {
-	lock.lock();
-	try {
-		_cairo_text_path(cr, utf8);
-	} finally {
-		lock.unlock();
-	}
-}
 /**
  * @param cr cast=(cairo_t *)
  * @param matrix cast=(cairo_matrix_t *)
@@ -1600,16 +1190,6 @@ public static final void cairo_translate(int /*long*/ cr, double tx, double ty)
 	}
 }
 /** @param cr cast=(cairo_t *) */
-public static final native void _cairo_user_to_device(int /*long*/ cr, double[] x, double[] y);
-public static final void cairo_user_to_device(int /*long*/ cr, double[] x, double[] y) {
-	lock.lock();
-	try {
-		_cairo_user_to_device(cr, x, y);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param cr cast=(cairo_t *) */
 public static final native void _cairo_user_to_device_distance(int /*long*/ cr, double[] dx, double[] dy);
 public static final void cairo_user_to_device_distance(int /*long*/ cr, double[] dx, double[] dy) {
 	lock.lock();
@@ -1634,43 +1214,6 @@ public static final int /*long*/ cairo_xlib_surface_create(int /*long*/ dpy, int
 		lock.unlock();
 	}
 }
-/**
- * @param dpy cast=(Display *)
- * @param pixmap cast=(Pixmap)
- * @param screen cast=(Screen *)
- */
-public static final native int /*long*/ _cairo_xlib_surface_create_for_bitmap(int /*long*/ dpy, int /*long*/ pixmap, int /*long*/ screen, int width, int height);
-public static final int /*long*/ cairo_xlib_surface_create_for_bitmap(int /*long*/ dpy, int /*long*/ pixmap, int /*long*/ screen, int width, int height) {
-	lock.lock();
-	try {
-		return _cairo_xlib_surface_create_for_bitmap(dpy, pixmap, screen, width, height);
-	} finally {
-		lock.unlock();
-	}
-}
-/** @param surface cast=(cairo_surface_t *) */
-public static final native void _cairo_xlib_surface_set_size(int /*long*/ surface, int width, int height);
-public static final void cairo_xlib_surface_set_size(int /*long*/ surface, int width, int height) {
-	lock.lock();
-	try {
-		_cairo_xlib_surface_set_size(surface, width, height);
-	} finally {
-		lock.unlock();
-	}
-}
-/**
- * @method flags=dynamic
- * @param surface cast=(cairo_surface_t *)
- */
-public static final native int /*long*/ _cairo_xlib_surface_get_drawable(int /*long*/ surface);
-public static final int /*long*/ cairo_xlib_surface_get_drawable(int /*long*/ surface) {
-	lock.lock();
-	try {
-		return _cairo_xlib_surface_get_drawable(surface);
-	} finally {
-		lock.unlock();
-	}
-}
 /**
  * @method flags=dynamic
  * @param surface cast=(cairo_surface_t *)
