@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
+ * Copyright (c) 2000, 2014 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -81,7 +81,7 @@ public Menu (Control parent) {
  * <p>
  * The style value is either one of the style constants defined in
  * class <code>SWT</code> which is applicable to instances of this
- * class, or must be built by <em>bitwise OR</em>'ing together 
+ * class, or must be built by <em>bitwise OR</em>'ing together
  * (that is, using the <code>int</code> "|" operator) two or more
  * of those <code>SWT</code> style constants. The class description
  * lists the style constants that are applicable to the class.
@@ -197,33 +197,29 @@ void _setVisible (boolean visible) {
 	if (visible) {
 		sendEvent (SWT.Show);
 		if (getItemCount () != 0) {
-			if ((OS.GTK_VERSION >=  OS.VERSION (2, 8, 0))) {
-				/*
-				* Feature in GTK. ON_TOP shells will send out 
-				* SWT.Deactivate whenever a context menu is shown.
-				* The fix is to prevent the menu from taking focus
-				* when it is being shown in an ON_TOP shell.
-				*/
-				if ((parent._getShell ().style & SWT.ON_TOP) != 0) {
-					OS.gtk_menu_shell_set_take_focus (handle, false);
-				}
+			/*
+			* Feature in GTK. ON_TOP shells will send out
+			* SWT.Deactivate whenever a context menu is shown.
+			* The fix is to prevent the menu from taking focus
+			* when it is being shown in an ON_TOP shell.
+			*/
+			if ((parent._getShell ().style & SWT.ON_TOP) != 0) {
+				OS.gtk_menu_shell_set_take_focus (handle, false);
 			}
 			int /*long*/ address = hasLocation ? display.menuPositionProc: 0;
 			hasLocation = false;
 			int /*long*/ data = 0;
-			if ((OS.GTK_VERSION >=  OS.VERSION (2, 10, 0))) {
-				/*
-				* Popup-menu to the status icon should be aligned to  
-				* Tray rather than to cursor position. There is a 
-				* possibility (unlikely) that TrayItem might have  
-				* been disposed in the listener, for which case  
-				* the menu should be shown in the cursor position. 
-				*/
-				TrayItem item = display.currentTrayItem;
-				if (item != null && !item.isDisposed()) {
-					 data = item.handle;
-					 address = OS.gtk_status_icon_position_menu_func ();
-				}
+			/*
+			* Popup-menu to the status icon should be aligned to
+			* Tray rather than to cursor position. There is a
+			* possibility (unlikely) that TrayItem might have
+			* been disposed in the listener, for which case
+			* the menu should be shown in the cursor position.
+			*/
+			TrayItem item = display.currentTrayItem;
+			if (item != null && !item.isDisposed()) {
+				 data = item.handle;
+				 address = OS.gtk_status_icon_position_menu_func ();
 			}
 			/*
 			* Bug in GTK.  The timestamp passed into gtk_menu_popup is used
@@ -301,6 +297,7 @@ public void addHelpListener (HelpListener listener) {
 	addListener (SWT.Help, typedListener);
 }
 
+@Override
 void createHandle (int index) {
 	state |= HANDLE;
 	if ((style & SWT.BAR) != 0) {
@@ -337,7 +334,7 @@ void createIMMenu (int /*long*/ imHandle) {
 	}
 	if (this.imHandle == imHandle) return;
 	this.imHandle = imHandle;
-	
+
 	if (imSeparator == 0) {
 		imSeparator = OS.gtk_separator_menu_item_new ();
 		OS.gtk_widget_show (imSeparator);
@@ -354,6 +351,7 @@ void createIMMenu (int /*long*/ imHandle) {
 	OS.gtk_menu_item_set_submenu (imItem, imSubmenu);
 }
 
+@Override
 void createWidget (int index) {
 	checkOrientation (parent);
 	super.createWidget (index);
@@ -379,7 +377,7 @@ void fixMenus (Decorations newParent) {
 	int [] origin_x = new int [1], origin_y = new int [1];
 	OS.gdk_window_get_origin (window, origin_x, origin_y);
 	GtkAllocation allocation = new GtkAllocation ();
-	gtk_widget_get_allocation (handle, allocation);
+	OS.gtk_widget_get_allocation (handle, allocation);
 	int x = origin_x [0] + allocation.x;
 	int y = origin_y [0] + allocation.y;
 	int width = allocation.width;
@@ -416,12 +414,12 @@ public MenuItem getDefaultItem () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * 
+ *
  * @see #isEnabled
  */
 public boolean getEnabled () {
 	checkWidget();
-	return gtk_widget_get_sensitive (handle);
+	return OS.gtk_widget_get_sensitive (handle);
 }
 
 /**
@@ -476,11 +474,11 @@ public int getItemCount () {
 
 /**
  * Returns a (possibly empty) array of <code>MenuItem</code>s which
- * are the items in the receiver. 
+ * are the items in the receiver.
  * <p>
  * Note: This is not the actual structure used by the receiver
  * to maintain its list of items, so modifying the array will
- * not affect the receiver. 
+ * not affect the receiver.
  * </p>
  *
  * @return the items in the receiver
@@ -494,25 +492,28 @@ public MenuItem [] getItems () {
 	checkWidget();
 	int /*long*/ list = OS.gtk_container_get_children (handle);
 	if (list == 0) return new MenuItem [0];
+	int /*long*/ originalList = list;
 	int count = OS.g_list_length (list);
 	if (imSeparator != 0) count--;
 	if (imItem != 0) count--;
 	MenuItem [] items = new MenuItem [count];
 	int index = 0;
 	for (int i=0; i<count; i++) {
-		int /*long*/ data = OS.g_list_nth_data (list, i);
+		int /*long*/ data = OS.g_list_data (list);
 		MenuItem item = (MenuItem) display.getWidget (data);
-		if (item != null) items [index++] = item; 
+		if (item != null) items [index++] = item;
+		list = OS.g_list_next (list);
 	}
-	OS.g_list_free (list);
+	OS.g_list_free (originalList);
 	if (index != items.length) {
-		MenuItem [] newItems = new MenuItem[index];
-		System.arraycopy(items, 0, newItems, 0, index);
+		MenuItem [] newItems = new MenuItem [index];
+		System.arraycopy (items, 0, newItems, 0, index);
 		items = newItems;
 	}
 	return items;
 }
 
+@Override
 String getNameText () {
 	String result = "";
 	MenuItem [] items = getItems ();
@@ -531,12 +532,12 @@ String getNameText () {
  * constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
  *
  * @return the orientation style
- * 
+ *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * 
+ *
  * @since 3.7
  */
 public int getOrientation () {
@@ -598,9 +599,10 @@ public Menu getParentMenu () {
  * Returns the receiver's shell. For all controls other than
  * shells, this simply returns the control's nearest ancestor
  * shell. Shells return themselves, even if they are children
- * of other shells.
+ * of other shells. Returns null if receiver or its ancestor 
+ * is the application menubar.
  *
- * @return the receiver's shell
+ * @return the receiver's shell or null
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -644,9 +646,13 @@ public boolean getVisible () {
 	return gtk_widget_get_mapped (handle);
 }
 
+@Override
 int /*long*/ gtk_hide (int /*long*/ widget) {
 	if ((style & SWT.POP_UP) != 0) {
-		if (display.activeShell != null) display.activeShell = getShell ();
+		if (display.activeShell != null) {
+			display.activeShell = getShell ();
+			display.activeShell.ignoreFocusOut = false;
+		}
 	}
 	sendEvent (SWT.Hide);
 	if (OS.ubuntu_menu_proxy_get() != 0) {
@@ -659,11 +665,15 @@ int /*long*/ gtk_hide (int /*long*/ widget) {
 	return 0;
 }
 
+@Override
 int /*long*/ gtk_show (int /*long*/ widget) {
 	if ((style & SWT.POP_UP) != 0) {
-		if (display.activeShell != null) display.activeShell = getShell ();
+		if (display.activeShell != null) {
+			display.activeShell = getShell ();
+			display.activeShell.ignoreFocusOut = true;
+		}
 		return 0;
-	} 
+	}
 	sendEvent (SWT.Show);
 	if (OS.ubuntu_menu_proxy_get() != 0) {
 		MenuItem[] items = getItems();
@@ -676,6 +686,7 @@ int /*long*/ gtk_show (int /*long*/ widget) {
 }
 
 
+@Override
 int /*long*/ gtk_show_help (int /*long*/ widget, int /*long*/ helpType) {
 	if (sendHelpEvent (helpType)) {
 		OS.gtk_menu_shell_deactivate (handle);
@@ -684,16 +695,17 @@ int /*long*/ gtk_show_help (int /*long*/ widget, int /*long*/ helpType) {
 	return 0;
 }
 
+@Override
 void hookEvents () {
 	super.hookEvents ();
-	OS.g_signal_connect_closure_by_id (handle, display.signalIds [SHOW], 0, display.closures [SHOW], false);
-	OS.g_signal_connect_closure_by_id (handle, display.signalIds [HIDE], 0, display.closures [HIDE], false);
-	OS.g_signal_connect_closure_by_id (handle, display.signalIds [SHOW_HELP], 0, display.closures [SHOW_HELP], false);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [SHOW], 0, display.getClosure (SHOW), false);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [HIDE], 0, display.getClosure (HIDE), false);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [SHOW_HELP], 0, display.getClosure (SHOW_HELP), false);
 }
 
 /**
  * Searches the receiver's list starting at the first item
- * (index 0) until an item is found that is equal to the 
+ * (index 0) until an item is found that is equal to the
  * argument, and returns the index of that item. If no item
  * is found, returns -1.
  *
@@ -730,7 +742,7 @@ public int indexOf (MenuItem item) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * 
+ *
  * @see #getEnabled
  */
 public boolean isEnabled () {
@@ -761,6 +773,7 @@ public boolean isVisible () {
 	return getVisible ();
 }
 
+@Override
 int /*long*/ menuPositionProc (int /*long*/ menu, int /*long*/ x, int /*long*/ y, int /*long*/ push_in, int /*long*/ user_data) {
 	/*
 	* Feature in GTK.  The menu position function sets the position of the
@@ -768,7 +781,7 @@ int /*long*/ menuPositionProc (int /*long*/ menu, int /*long*/ x, int /*long*/ y
 	* add a scroll arrow at the bottom and position the first menu entry at
 	* the specified position.  The fix is to flip the menu location to be
 	* completely inside the screen.
-	* 
+	*
 	* NOTE: This code doesn't work for multiple monitors.
 	*/
     GtkRequisition requisition = new GtkRequisition ();
@@ -777,7 +790,7 @@ int /*long*/ menuPositionProc (int /*long*/ menu, int /*long*/ x, int /*long*/ y
 	int reqy = this.y;
 	if (reqy + requisition.height > screenHeight) {
     	reqy = Math.max (0, reqy - requisition.height);
-	} 
+	}
     int screenWidth = OS.gdk_screen_width ();
 	int reqx = this.x;
     if ((style & SWT.RIGHT_TO_LEFT) != 0) {
@@ -791,6 +804,7 @@ int /*long*/ menuPositionProc (int /*long*/ menu, int /*long*/ x, int /*long*/ y
 	return 0;
 }
 
+@Override
 void releaseChildren (boolean destroy) {
 	MenuItem [] items = getItems ();
 	for (int i=0; i<items.length; i++) {
@@ -802,6 +816,7 @@ void releaseChildren (boolean destroy) {
 	super.releaseChildren (destroy);
 }
 
+@Override
 void releaseParent () {
 	super.releaseParent ();
 	if (cascade != null) cascade.setMenu (null);
@@ -814,9 +829,10 @@ void releaseParent () {
 	}
 }
 
+@Override
 void releaseWidget () {
 	super.releaseWidget ();
-	if (parent != null) parent.removeMenu (this); 
+	if (parent != null) parent.removeMenu (this);
 	parent = null;
 	cascade = null;
 	imItem = imSeparator = imHandle = 0;
@@ -881,6 +897,7 @@ public void removeHelpListener (HelpListener listener) {
 	eventTable.unhook (SWT.Help, listener);
 }
 
+@Override
 void reskinChildren (int flags) {
 	MenuItem [] items = getItems ();
 	for (int i=0; i<items.length; i++) {
@@ -907,11 +924,11 @@ boolean sendHelpEvent (int /*long*/ helpType) {
 /**
  * Sets the default menu item to the argument or removes
  * the default emphasis when the argument is <code>null</code>.
- * 
+ *
  * @param item the default menu item or null
  *
  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_INVALID_ARGUMENT - if the menu item has been disposed</li> 
+ *    <li>ERROR_INVALID_ARGUMENT - if the menu item has been disposed</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -965,7 +982,7 @@ public void setLocation (int x, int y) {
 	if ((style & (SWT.BAR | SWT.DROP_DOWN)) != 0) return;
 	this.x = x;
 	this.y = y;
-	hasLocation = true;	
+	hasLocation = true;
 }
 
 /**
@@ -989,7 +1006,7 @@ public void setLocation (int x, int y) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * 
+ *
  * @since 2.1
  */
 public void setLocation (Point location) {
@@ -1004,16 +1021,16 @@ public void setLocation (Point location) {
  * <p>
  *
  * @param orientation new orientation style
- * 
+ *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * 
- * @since 3.7  
+ *
+ * @since 3.7
  */
 public void setOrientation (int orientation) {
-    checkWidget ();    
+    checkWidget ();
     if ((style & (SWT.BAR | SWT.DROP_DOWN)) != 0) return;
     _setOrientation (orientation);
 }
@@ -1026,6 +1043,7 @@ void _setOrientation (int orientation) {
     setOrientation (false);
 }
 
+@Override
 void setOrientation (boolean create) {
     if ((style & SWT.RIGHT_TO_LEFT) != 0 || !create) {
     	int dir = (style & SWT.RIGHT_TO_LEFT) != 0 ? OS.GTK_TEXT_DIR_RTL : OS.GTK_TEXT_DIR_LTR;
@@ -1039,7 +1057,7 @@ void setOrientation (boolean create) {
 
 /**
  * Marks the receiver as visible if the argument is <code>true</code>,
- * and marks it invisible otherwise. 
+ * and marks it invisible otherwise.
  * <p>
  * If one of the receiver's ancestors is not visible or some
  * other condition makes the receiver not visible, marking
