@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2012 IBM Corporation and others.
+ * Copyright (c) 2000, 2014 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -50,7 +50,7 @@ public class TableColumn extends Item {
  * <p>
  * The style value is either one of the style constants defined in
  * class <code>SWT</code> which is applicable to instances of this
- * class, or must be built by <em>bitwise OR</em>'ing together 
+ * class, or must be built by <em>bitwise OR</em>'ing together
  * (that is, using the <code>int</code> "|" operator) two or more
  * of those <code>SWT</code> style constants. The class description
  * lists the style constants that are applicable to the class.
@@ -88,7 +88,7 @@ public TableColumn (Table parent, int style) {
  * <p>
  * The style value is either one of the style constants defined in
  * class <code>SWT</code> which is applicable to instances of this
- * class, or must be built by <em>bitwise OR</em>'ing together 
+ * class, or must be built by <em>bitwise OR</em>'ing together
  * (that is, using the <code>int</code> "|" operator) two or more
  * of those <code>SWT</code> style constants. The class description
  * lists the style constants that are applicable to the class.
@@ -186,10 +186,12 @@ static int checkStyle (int style) {
 	return checkBits (style, SWT.LEFT, SWT.CENTER, SWT.RIGHT, 0, 0, 0);
 }
 
+@Override
 protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 
+@Override
 void createWidget (int index) {
 	parent.createItem (this, index);
 	setOrientation (true);
@@ -198,6 +200,7 @@ void createWidget (int index) {
 	text = "";
 }
 
+@Override
 void deregister() {
 	super.deregister ();
 	display.removeWidget (handle);
@@ -205,6 +208,7 @@ void deregister() {
 	if (labelHandle != 0) display.removeWidget (labelHandle);
 }
 
+@Override
 void destroyWidget () {
 	parent.destroyItem (this);
 	releaseHandle ();
@@ -215,7 +219,7 @@ void destroyWidget () {
  * text or image in the receiver. The value will be one of
  * <code>LEFT</code>, <code>RIGHT</code> or <code>CENTER</code>.
  *
- * @return the alignment 
+ * @return the alignment
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -232,8 +236,8 @@ public int getAlignment () {
 
 /**
  * Gets the moveable attribute. A column that is
- * not moveable cannot be reordered by the user 
- * by dragging the header but may be reordered 
+ * not moveable cannot be reordered by the user
+ * by dragging the header but may be reordered
  * by the programmer.
  *
  * @return the moveable attribute
@@ -242,12 +246,12 @@ public int getAlignment () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * 
+ *
  * @see Table#getColumnOrder()
  * @see Table#setColumnOrder(int[])
  * @see TableColumn#setMoveable(boolean)
  * @see SWT#Move
- * 
+ *
  * @since 3.1
  */
 public boolean getMoveable() {
@@ -297,7 +301,7 @@ public boolean getResizable () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * 
+ *
  * @since 3.2
  */
 public String getToolTipText () {
@@ -324,6 +328,7 @@ public int getWidth () {
 	return OS.gtk_tree_view_column_get_width (handle);
 }
 
+@Override
 int /*long*/ gtk_clicked (int /*long*/ widget) {
 	/*
 	* There is no API to get a double click on a table column.  Normally, when
@@ -356,6 +361,7 @@ int /*long*/ gtk_clicked (int /*long*/ widget) {
 	return 0;
 }
 
+@Override
 int /*long*/ gtk_event_after (int /*long*/ widget, int /*long*/ gdkEvent) {
 	GdkEvent event = new GdkEvent ();
 	OS.memmove (event, gdkEvent, GdkEvent.sizeof);
@@ -372,14 +378,16 @@ int /*long*/ gtk_event_after (int /*long*/ widget, int /*long*/ gdkEvent) {
 	return 0;
 }
 
+@Override
 int /*long*/ gtk_mnemonic_activate (int /*long*/ widget, int /*long*/ arg1) {
 	return parent.gtk_mnemonic_activate (widget, arg1);
 }
 
+@Override
 int /*long*/ gtk_size_allocate (int /*long*/ widget, int /*long*/ allocation) {
 	useFixedWidth = false;
 	GtkAllocation widgetAllocation = new GtkAllocation ();
-	gtk_widget_get_allocation (widget, widgetAllocation);
+	OS.gtk_widget_get_allocation (widget, widgetAllocation);
 	int x = widgetAllocation.x;
 	int width = widgetAllocation.width;
 	if (x != lastX) {
@@ -393,14 +401,15 @@ int /*long*/ gtk_size_allocate (int /*long*/ widget, int /*long*/ allocation) {
 	return 0;
 }
 
+@Override
 void hookEvents () {
 	super.hookEvents ();
-	OS.g_signal_connect_closure (handle, OS.clicked, display.closures [CLICKED], false);
+	OS.g_signal_connect_closure (handle, OS.clicked, display.getClosure (CLICKED), false);
 	if (buttonHandle != 0) {
-		OS.g_signal_connect_closure_by_id (buttonHandle, display.signalIds [SIZE_ALLOCATE], 0, display.closures [SIZE_ALLOCATE], false);
-		OS.g_signal_connect_closure_by_id (buttonHandle, display.signalIds [EVENT_AFTER], 0, display.closures [EVENT_AFTER], false);
+		OS.g_signal_connect_closure_by_id (buttonHandle, display.signalIds [SIZE_ALLOCATE], 0, display.getClosure (SIZE_ALLOCATE), false);
+		OS.g_signal_connect_closure_by_id (buttonHandle, display.signalIds [EVENT_AFTER], 0, display.getClosure (EVENT_AFTER), false);
 	}
-	if (labelHandle != 0) OS.g_signal_connect_closure_by_id (labelHandle, display.signalIds [MNEMONIC_ACTIVATE], 0, display.closures [MNEMONIC_ACTIVATE], false);
+	if (labelHandle != 0) OS.g_signal_connect_closure_by_id (labelHandle, display.signalIds [MNEMONIC_ACTIVATE], 0, display.getClosure (MNEMONIC_ACTIVATE), false);
 }
 
 /**
@@ -441,6 +450,7 @@ public void pack () {
 	setWidth(width);
 }
 
+@Override
 void register () {
 	super.register ();
 	display.addWidget (handle, this);
@@ -448,13 +458,15 @@ void register () {
 	if (labelHandle != 0) display.addWidget (labelHandle, this);
 }
 
+@Override
 void releaseHandle () {
 	super.releaseHandle ();
 	handle = buttonHandle = labelHandle = imageHandle = 0;
-	modelIndex = -1; 
+	modelIndex = -1;
 	parent = null;
 }
 
+@Override
 void releaseParent () {
 	super.releaseParent ();
 	if (parent.sortColumn == this) {
@@ -509,7 +521,7 @@ public void removeSelectionListener(SelectionListener listener) {
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (eventTable == null) return;
 	eventTable.unhook (SWT.Selection, listener);
-	eventTable.unhook (SWT.DefaultSelection,listener);	
+	eventTable.unhook (SWT.DefaultSelection,listener);
 }
 
 /**
@@ -520,7 +532,7 @@ public void removeSelectionListener(SelectionListener listener) {
  * Note that due to a restriction on some platforms, the first column
  * is always left aligned.
  * </p>
- * @param alignment the new alignment 
+ * @param alignment the new alignment
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -542,6 +554,7 @@ void setFontDescription (int /*long*/ font) {
 	setFontDescription (imageHandle, font);
 }
 
+@Override
 public void setImage (Image image) {
 	checkWidget ();
 	super.setImage (image);
@@ -564,8 +577,8 @@ public void setImage (Image image) {
 /**
  * Sets the resizable attribute.  A column that is
  * resizable can be resized by the user dragging the
- * edge of the header.  A column that is not resizable 
- * cannot be dragged by the user but may be resized 
+ * edge of the header.  A column that is not resizable
+ * cannot be dragged by the user but may be resized
  * by the programmer.
  *
  * @param resizable the resize attribute
@@ -583,8 +596,8 @@ public void setResizable (boolean resizable) {
 /**
  * Sets the moveable attribute.  A column that is
  * moveable can be reordered by the user by dragging
- * the header. A column that is not moveable cannot be 
- * dragged by the user but may be reordered 
+ * the header. A column that is not moveable cannot be
+ * dragged by the user but may be reordered
  * by the programmer.
  *
  * @param moveable the moveable attribute
@@ -593,12 +606,12 @@ public void setResizable (boolean resizable) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * 
+ *
  * @see Table#setColumnOrder(int[])
  * @see Table#getColumnOrder()
  * @see TableColumn#getMoveable()
  * @see SWT#Move
- * 
+ *
  * @since 3.1
  */
 public void setMoveable (boolean moveable) {
@@ -606,6 +619,7 @@ public void setMoveable (boolean moveable) {
 	OS.gtk_tree_view_column_set_reorderable (handle, moveable);
 }
 
+@Override
 void setOrientation (boolean create) {
 	if ((parent.style & SWT.RIGHT_TO_LEFT) != 0 || !create) {
 		if (buttonHandle != 0) {
@@ -616,6 +630,7 @@ void setOrientation (boolean create) {
 	}
 }
 
+@Override
 public void setText (String string) {
 	checkWidget();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
@@ -632,24 +647,24 @@ public void setText (String string) {
 
 /**
  * Sets the receiver's tool tip text to the argument, which
- * may be null indicating that the default tool tip for the 
+ * may be null indicating that the default tool tip for the
  * control will be shown. For a control that has a default
  * tool tip, such as the Tree control on Windows, setting
  * the tool tip text to an empty string replaces the default,
  * causing no tool tip text to be shown.
  * <p>
  * The mnemonic indicator (character '&amp;') is not displayed in a tool tip.
- * To display a single '&amp;' in the tool tip, the character '&amp;' can be 
+ * To display a single '&amp;' in the tool tip, the character '&amp;' can be
  * escaped by doubling it in the string.
  * </p>
- * 
+ *
  * @param string the new tool tip text (or null)
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- * 
+ *
  * @since 3.2
  */
 public void setToolTipText (String string) {
@@ -691,10 +706,10 @@ public void setWidth (int width) {
 	lastWidth = width;
 	/*
 	 * Bug in GTK. When the column is made visible the event window of column
-	 * header is raised above the gripper window of the previous column. In 
+	 * header is raised above the gripper window of the previous column. In
 	 * some cases, this can cause the previous column to be not resizable by
 	 * the mouse. The fix is to find the event window and lower it to bottom to
-	 * the z-order stack. 
+	 * the z-order stack.
 	 */
 	if (width != 0) {
 		if (buttonHandle != 0) {
