@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others. All rights reserved.
+ * Copyright (c) 2000, 2015 IBM Corporation and others. All rights reserved.
  * The contents of this file are made available under the terms
  * of the GNU Lesser General Public License (LGPL) Version 2.1 that
  * accompanies this distribution (lgpl-v21.txt).  The LGPL is also
@@ -41,6 +41,8 @@ typedef enum {
 #endif
 	GDK_1EVENT_1TYPE_FUNC,
 	GDK_1EVENT_1WINDOW_FUNC,
+	GDK_1IS_1X11_1DISPLAY_FUNC,
+	GDK_1WINDOWING_1WAYLAND_FUNC,
 	GDK_1WINDOWING_1X11_FUNC,
 	GInterfaceInfo_1sizeof_FUNC,
 	GPollFD_1sizeof_FUNC,
@@ -49,23 +51,11 @@ typedef enum {
 	GTK_1ENTRY_1IM_1CONTEXT_FUNC,
 	GTK_1RANGE_1SLIDER_1END_FUNC,
 	GTK_1RANGE_1SLIDER_1START_FUNC,
-	GTK_1SCROLLED_1WINDOW_1HSCROLLBAR_FUNC,
 	GTK_1SCROLLED_1WINDOW_1SCROLLBAR_1SPACING_FUNC,
-	GTK_1SCROLLED_1WINDOW_1VSCROLLBAR_FUNC,
 	GTK_1TEXTVIEW_1IM_1CONTEXT_FUNC,
-	GTK_1TOOLTIPS_1GET_1TIP_1TEXT_FUNC,
-	GTK_1TOOLTIPS_1SET_1ACTIVE_FUNC,
-	GTK_1TOOLTIPS_1TIP_1WINDOW_FUNC,
 	GTK_1TYPE_1ACCESSIBLE_FUNC,
-	GTK_1WIDGET_1HEIGHT_FUNC,
 	GTK_1WIDGET_1REQUISITION_1HEIGHT_FUNC,
 	GTK_1WIDGET_1REQUISITION_1WIDTH_FUNC,
-	GTK_1WIDGET_1SET_1X_FUNC,
-	GTK_1WIDGET_1SET_1Y_FUNC,
-	GTK_1WIDGET_1WIDTH_FUNC,
-	GTK_1WIDGET_1WINDOW_FUNC,
-	GTK_1WIDGET_1X_FUNC,
-	GTK_1WIDGET_1Y_FUNC,
 	GTypeInfo_1sizeof_FUNC,
 	GTypeQuery_1sizeof_FUNC,
 	G_1OBJECT_1CLASS_1CONSTRUCTOR_FUNC,
@@ -87,7 +77,6 @@ typedef enum {
 	GdkEventMotion_1sizeof_FUNC,
 	GdkEventProperty_1sizeof_FUNC,
 	GdkEventScroll_1sizeof_FUNC,
-	GdkEventVisibility_1sizeof_FUNC,
 	GdkEventWindowState_1sizeof_FUNC,
 	GdkEvent_1sizeof_FUNC,
 	GdkGeometry_1sizeof_FUNC,
@@ -103,9 +92,7 @@ typedef enum {
 	GtkCellRendererText_1sizeof_FUNC,
 	GtkCellRendererToggleClass_1sizeof_FUNC,
 	GtkCellRendererToggle_1sizeof_FUNC,
-	GtkColorSelectionDialog_1sizeof_FUNC,
 	GtkRequisition_1sizeof_FUNC,
-	GtkSelectionData_1sizeof_FUNC,
 	GtkTargetEntry_1sizeof_FUNC,
 	GtkTextIter_1sizeof_FUNC,
 	GtkTreeIter_1sizeof_FUNC,
@@ -121,14 +108,10 @@ typedef enum {
 	RTLD_1LAZY_FUNC,
 	RTLD_1NOW_FUNC,
 	XAnyEvent_1sizeof_FUNC,
-	XClientMessageEvent_1sizeof_FUNC,
-	XCrossingEvent_1sizeof_FUNC,
 	XEvent_1sizeof_FUNC,
 	XExposeEvent_1sizeof_FUNC,
 	XFocusChangeEvent_1sizeof_FUNC,
 	XRenderPictureAttributes_1sizeof_FUNC,
-	XVisibilityEvent_1sizeof_FUNC,
-	XWindowChanges_1sizeof_FUNC,
 	X_1EVENT_1TYPE_FUNC,
 	X_1EVENT_1WINDOW_FUNC,
 	_1Call_FUNC,
@@ -162,13 +145,9 @@ typedef enum {
 	_1GTK_1TYPE_1MENU_FUNC,
 	_1GTK_1TYPE_1WIDGET_FUNC,
 	_1GTK_1WIDGET_1FLAGS_FUNC,
-	_1GTK_1WIDGET_1HAS_1DEFAULT_FUNC,
-	_1GTK_1WIDGET_1HAS_1FOCUS_FUNC,
 	_1GTK_1WIDGET_1MAPPED_FUNC,
-	_1GTK_1WIDGET_1SENSITIVE_FUNC,
 	_1GTK_1WIDGET_1SET_1FLAGS_FUNC,
 	_1GTK_1WIDGET_1UNSET_1FLAGS_FUNC,
-	_1GTK_1WIDGET_1VISIBLE_FUNC,
 	_1G_1OBJECT_1CLASS_FUNC,
 	_1G_1OBJECT_1GET_1CLASS_FUNC,
 	_1G_1OBJECT_1TYPE_FUNC,
@@ -177,6 +156,8 @@ typedef enum {
 	_1G_1TYPE_1STRING_FUNC,
 	_1PANGO_1PIXELS_FUNC,
 	_1PANGO_1TYPE_1FONT_1DESCRIPTION_FUNC,
+	_1PANGO_1TYPE_1FONT_1FACE_FUNC,
+	_1PANGO_1TYPE_1FONT_1FAMILY_FUNC,
 	_1PANGO_1TYPE_1LAYOUT_FUNC,
 	_1XCheckIfEvent_FUNC,
 	_1XDefaultRootWindow_FUNC,
@@ -189,7 +170,6 @@ typedef enum {
 	_1XKeysymToKeycode_FUNC,
 	_1XListProperties_FUNC,
 	_1XQueryPointer_FUNC,
-	_1XReconfigureWMWindow_FUNC,
 	_1XRenderComposite_FUNC,
 	_1XRenderCreatePicture_FUNC,
 	_1XRenderFindStandardFormat_FUNC,
@@ -199,7 +179,6 @@ typedef enum {
 	_1XRenderQueryVersion_FUNC,
 	_1XRenderSetPictureClipRectangles_FUNC,
 	_1XRenderSetPictureTransform_FUNC,
-	_1XSendEvent_FUNC,
 	_1XSetErrorHandler_FUNC,
 	_1XSetIOErrorHandler_FUNC,
 	_1XSetInputFocus_FUNC,
@@ -242,13 +221,27 @@ typedef enum {
 	_1g_1app_1info_1launch_1default_1for_1uri_FUNC,
 	_1g_1app_1info_1should_1show_FUNC,
 	_1g_1app_1info_1supports_1uris_FUNC,
+	_1g_1bus_1get_1sync_FUNC,
+	_1g_1bus_1own_1name_FUNC,
 	_1g_1cclosure_1new_FUNC,
 	_1g_1closure_1ref_FUNC,
+	_1g_1closure_1sink_FUNC,
 	_1g_1closure_1unref_FUNC,
 	_1g_1content_1type_1equals_FUNC,
 	_1g_1content_1type_1is_1a_FUNC,
 	_1g_1data_1input_1stream_1new_FUNC,
 	_1g_1data_1input_1stream_1read_1line_FUNC,
+	_1g_1dbus_1connection_1register_1object_FUNC,
+	_1g_1dbus_1method_1invocation_1return_1value_FUNC,
+	_1g_1dbus_1node_1info_1lookup_1interface_FUNC,
+	_1g_1dbus_1node_1info_1new_1for_1xml_FUNC,
+	_1g_1dbus_1proxy_1call_FUNC,
+	_1g_1dbus_1proxy_1call_1finish_FUNC,
+	_1g_1dbus_1proxy_1call_1sync_FUNC,
+	_1g_1dbus_1proxy_1new_FUNC,
+	_1g_1dbus_1proxy_1new_1finish_FUNC,
+	_1g_1dbus_1proxy_1new_1sync_FUNC,
+	_1g_1error_1get_1message_FUNC,
 	_1g_1file_1get_1uri_FUNC,
 	_1g_1file_1info_1get_1content_1type_FUNC,
 	_1g_1file_1info_1get_1modification_1time_FUNC,
@@ -264,6 +257,7 @@ typedef enum {
 	_1g_1filename_1to_1uri_FUNC,
 	_1g_1filename_1to_1utf8_FUNC,
 	_1g_1free_FUNC,
+	_1g_1hash_1table_1get_1values_FUNC,
 	_1g_1icon_1new_1for_1string_FUNC,
 	_1g_1icon_1to_1string_FUNC,
 	_1g_1idle_1add_FUNC,
@@ -329,6 +323,11 @@ typedef enum {
 #else
 	_1g_1object_1set__J_3BLorg_eclipse_swt_internal_gtk_GdkColor_2J_FUNC,
 #endif
+#ifndef JNI64
+	_1g_1object_1set__I_3BLorg_eclipse_swt_internal_gtk_GdkRGBA_2I_FUNC,
+#else
+	_1g_1object_1set__J_3BLorg_eclipse_swt_internal_gtk_GdkRGBA_2J_FUNC,
+#endif
 #ifndef JNI64
 	_1g_1object_1set__I_3BZI_FUNC,
 #else
@@ -421,6 +420,23 @@ typedef enum {
 	_1g_1utf8_1to_1utf16___3BJ_3J_3J_3J_FUNC,
 #endif
 	_1g_1value_1peek_1pointer_FUNC,
+	_1g_1variant_1get_1boolean_FUNC,
+	_1g_1variant_1get_1child_1value_FUNC,
+	_1g_1variant_1get_1double_FUNC,
+	_1g_1variant_1get_1string_FUNC,
+	_1g_1variant_1get_1type_FUNC,
+	_1g_1variant_1get_1type_1string_FUNC,
+	_1g_1variant_1get_1uint64_FUNC,
+	_1g_1variant_1is_1of_1type_FUNC,
+	_1g_1variant_1n_1children_FUNC,
+	_1g_1variant_1new_1boolean_FUNC,
+	_1g_1variant_1new_1byte_FUNC,
+	_1g_1variant_1new_1double_FUNC,
+	_1g_1variant_1new_1int64_FUNC,
+	_1g_1variant_1new_1maybe_FUNC,
+	_1g_1variant_1new_1string_FUNC,
+	_1g_1variant_1new_1tuple_FUNC,
+	_1g_1variant_1new_1uint64_FUNC,
 	_1gdk_1atom_1intern_FUNC,
 	_1gdk_1atom_1name_FUNC,
 	_1gdk_1beep_FUNC,
@@ -448,6 +464,7 @@ typedef enum {
 	_1gdk_1device_1grab_FUNC,
 	_1gdk_1device_1manager_1get_1client_1pointer_FUNC,
 	_1gdk_1device_1ungrab_FUNC,
+	_1gdk_1device_1warp_FUNC,
 	_1gdk_1display_1get_1default_FUNC,
 	_1gdk_1display_1get_1device_1manager_FUNC,
 	_1gdk_1display_1supports_1cursor_1color_FUNC,
@@ -551,8 +568,10 @@ typedef enum {
 	_1gdk_1screen_1get_1monitor_1at_1point_FUNC,
 	_1gdk_1screen_1get_1monitor_1at_1window_FUNC,
 	_1gdk_1screen_1get_1monitor_1geometry_FUNC,
+	_1gdk_1screen_1get_1monitor_1width_1mm_FUNC,
+	_1gdk_1screen_1get_1monitor_1workarea_FUNC,
 	_1gdk_1screen_1get_1n_1monitors_FUNC,
-	_1gdk_1screen_1get_1number_FUNC,
+	_1gdk_1screen_1get_1primary_1monitor_FUNC,
 	_1gdk_1screen_1height_FUNC,
 	_1gdk_1screen_1width_FUNC,
 	_1gdk_1screen_1width_1mm_FUNC,
@@ -566,7 +585,6 @@ typedef enum {
 	_1gdk_1window_1add_1filter_FUNC,
 	_1gdk_1window_1at_1pointer_FUNC,
 	_1gdk_1window_1begin_1paint_1rect_FUNC,
-	_1gdk_1window_1begin_1paint_1region_FUNC,
 	_1gdk_1window_1create_1similar_1surface_FUNC,
 	_1gdk_1window_1destroy_FUNC,
 	_1gdk_1window_1end_1paint_FUNC,
@@ -607,6 +625,7 @@ typedef enum {
 	_1gdk_1window_1set_1debug_1updates_FUNC,
 	_1gdk_1window_1set_1decorations_FUNC,
 	_1gdk_1window_1set_1events_FUNC,
+	_1gdk_1window_1set_1functions_FUNC,
 	_1gdk_1window_1set_1override_1redirect_FUNC,
 	_1gdk_1window_1set_1user_1data_FUNC,
 	_1gdk_1window_1shape_1combine_1region_FUNC,
@@ -627,11 +646,11 @@ typedef enum {
 	_1glib_1micro_1version_FUNC,
 	_1glib_1minor_1version_FUNC,
 	_1gtk_1accel_1group_1new_FUNC,
+	_1gtk_1accel_1label_1set_1accel_FUNC,
 	_1gtk_1accel_1label_1set_1accel_1widget_FUNC,
 	_1gtk_1accelerator_1get_1default_1mod_1mask_FUNC,
 	_1gtk_1accelerator_1parse_FUNC,
 	_1gtk_1accessible_1get_1widget_FUNC,
-	_1gtk_1adjustment_1changed_FUNC,
 	_1gtk_1adjustment_1configure_FUNC,
 	_1gtk_1adjustment_1get_1lower_FUNC,
 	_1gtk_1adjustment_1get_1page_1increment_FUNC,
@@ -705,6 +724,7 @@ typedef enum {
 	_1gtk_1combo_1box_1remove_1text_FUNC,
 	_1gtk_1combo_1box_1set_1active_FUNC,
 	_1gtk_1combo_1box_1set_1focus_1on_1click_FUNC,
+	_1gtk_1combo_1box_1set_1wrap_1width_FUNC,
 	_1gtk_1combo_1box_1text_1insert_FUNC,
 	_1gtk_1combo_1box_1text_1new_FUNC,
 	_1gtk_1combo_1box_1text_1new_1with_1entry_FUNC,
@@ -722,6 +742,7 @@ typedef enum {
 	_1gtk_1dialog_1add_1button_FUNC,
 	_1gtk_1dialog_1run_FUNC,
 	_1gtk_1drag_1begin_FUNC,
+	_1gtk_1drag_1begin_1with_1coordinates_FUNC,
 	_1gtk_1drag_1check_1threshold_FUNC,
 	_1gtk_1drag_1dest_1find_1target_FUNC,
 	_1gtk_1drag_1dest_1set_FUNC,
@@ -753,7 +774,7 @@ typedef enum {
 	_1gtk_1entry_1new_FUNC,
 	_1gtk_1entry_1set_1alignment_FUNC,
 	_1gtk_1entry_1set_1has_1frame_FUNC,
-	_1gtk_1entry_1set_1icon_1from_1stock_FUNC,
+	_1gtk_1entry_1set_1icon_1from_1icon_1name_FUNC,
 	_1gtk_1entry_1set_1icon_1sensitive_FUNC,
 	_1gtk_1entry_1set_1invisible_1char_FUNC,
 	_1gtk_1entry_1set_1max_1length_FUNC,
@@ -790,7 +811,6 @@ typedef enum {
 	_1gtk_1file_1filter_1set_1name_FUNC,
 	_1gtk_1fixed_1move_FUNC,
 	_1gtk_1fixed_1new_FUNC,
-	_1gtk_1fixed_1set_1has_1window_FUNC,
 	_1gtk_1font_1chooser_1dialog_1new_FUNC,
 	_1gtk_1font_1chooser_1get_1font_FUNC,
 	_1gtk_1font_1chooser_1set_1font_FUNC,
@@ -834,8 +854,10 @@ typedef enum {
 	_1gtk_1image_1menu_1item_1new_1with_1label_FUNC,
 	_1gtk_1image_1menu_1item_1set_1image_FUNC,
 	_1gtk_1image_1new_FUNC,
+	_1gtk_1image_1new_1from_1icon_1name_FUNC,
 	_1gtk_1image_1new_1from_1pixbuf_FUNC,
 	_1gtk_1image_1set_1from_1gicon_FUNC,
+	_1gtk_1image_1set_1from_1icon_1name_FUNC,
 	_1gtk_1image_1set_1from_1pixbuf_FUNC,
 	_1gtk_1init_1check_FUNC,
 	_1gtk_1label_1get_1layout_FUNC,
@@ -857,6 +879,8 @@ typedef enum {
 	_1gtk_1label_1set_1text__J_3B_FUNC,
 #endif
 	_1gtk_1label_1set_1text_1with_1mnemonic_FUNC,
+	_1gtk_1label_1set_1xalign_FUNC,
+	_1gtk_1label_1set_1yalign_FUNC,
 	_1gtk_1list_1store_1append_FUNC,
 	_1gtk_1list_1store_1clear_FUNC,
 	_1gtk_1list_1store_1insert_FUNC,
@@ -892,7 +916,6 @@ typedef enum {
 	_1gtk_1major_1version_FUNC,
 	_1gtk_1menu_1bar_1new_FUNC,
 	_1gtk_1menu_1item_1get_1submenu_FUNC,
-	_1gtk_1menu_1item_1remove_1submenu_FUNC,
 	_1gtk_1menu_1item_1set_1submenu_FUNC,
 	_1gtk_1menu_1new_FUNC,
 	_1gtk_1menu_1popdown_FUNC,
@@ -916,7 +939,6 @@ typedef enum {
 	_1gtk_1notebook_1set_1scrollable_FUNC,
 	_1gtk_1notebook_1set_1show_1tabs_FUNC,
 	_1gtk_1notebook_1set_1tab_1pos_FUNC,
-	_1gtk_1object_1sink_FUNC,
 	_1gtk_1orientable_1set_1orientation_FUNC,
 	_1gtk_1page_1setup_1get_1bottom_1margin_FUNC,
 	_1gtk_1page_1setup_1get_1left_1margin_FUNC,
@@ -960,7 +982,6 @@ typedef enum {
 	_1gtk_1paper_1size_1new_FUNC,
 	_1gtk_1paper_1size_1new_1custom_FUNC,
 	_1gtk_1paper_1size_1new_1from_1ppd_FUNC,
-	_1gtk_1plug_1get_1id_FUNC,
 	_1gtk_1plug_1new_FUNC,
 	_1gtk_1print_1job_1get_1surface_FUNC,
 	_1gtk_1print_1job_1new_FUNC,
@@ -995,6 +1016,7 @@ typedef enum {
 	_1gtk_1printer_1get_1backend_FUNC,
 	_1gtk_1printer_1get_1name_FUNC,
 	_1gtk_1printer_1is_1default_FUNC,
+	_1gtk_1printer_1option_1widget_1get_1type_FUNC,
 	_1gtk_1progress_1bar_1new_FUNC,
 	_1gtk_1progress_1bar_1pulse_FUNC,
 	_1gtk_1progress_1bar_1set_1fraction_FUNC,
@@ -1068,6 +1090,7 @@ typedef enum {
 	_1gtk_1spin_1button_1new_FUNC,
 	_1gtk_1spin_1button_1set_1digits_FUNC,
 	_1gtk_1spin_1button_1set_1increments_FUNC,
+	_1gtk_1spin_1button_1set_1numeric_FUNC,
 	_1gtk_1spin_1button_1set_1range_FUNC,
 	_1gtk_1spin_1button_1set_1value_FUNC,
 	_1gtk_1spin_1button_1set_1wrap_FUNC,
@@ -1137,9 +1160,9 @@ typedef enum {
 #else
 	_1gtk_1text_1buffer_1insert__J_3B_3BI_FUNC,
 #endif
-	_1gtk_1text_1buffer_1move_1mark_FUNC,
 	_1gtk_1text_1buffer_1paste_1clipboard_FUNC,
 	_1gtk_1text_1buffer_1place_1cursor_FUNC,
+	_1gtk_1text_1buffer_1select_1range_FUNC,
 	_1gtk_1text_1buffer_1set_1text_FUNC,
 	_1gtk_1text_1iter_1get_1line_FUNC,
 	_1gtk_1text_1iter_1get_1offset_FUNC,
@@ -1178,15 +1201,8 @@ typedef enum {
 	_1gtk_1toolbar_1insert_FUNC,
 	_1gtk_1toolbar_1new_FUNC,
 	_1gtk_1toolbar_1set_1icon_1size_FUNC,
-	_1gtk_1toolbar_1set_1orientation_FUNC,
 	_1gtk_1toolbar_1set_1show_1arrow_FUNC,
 	_1gtk_1toolbar_1set_1style_FUNC,
-	_1gtk_1tooltips_1data_1get_FUNC,
-	_1gtk_1tooltips_1disable_FUNC,
-	_1gtk_1tooltips_1enable_FUNC,
-	_1gtk_1tooltips_1force_1window_FUNC,
-	_1gtk_1tooltips_1new_FUNC,
-	_1gtk_1tooltips_1set_1tip_FUNC,
 #ifndef JNI64
 	_1gtk_1tree_1model_1get__III_3II_FUNC,
 #else
@@ -1266,7 +1282,6 @@ typedef enum {
 	_1gtk_1tree_1view_1column_1cell_1set_1cell_1data_FUNC,
 	_1gtk_1tree_1view_1column_1clear_FUNC,
 	_1gtk_1tree_1view_1column_1get_1button_FUNC,
-	_1gtk_1tree_1view_1column_1get_1cell_1renderers_FUNC,
 	_1gtk_1tree_1view_1column_1get_1fixed_1width_FUNC,
 	_1gtk_1tree_1view_1column_1get_1reorderable_FUNC,
 	_1gtk_1tree_1view_1column_1get_1resizable_FUNC,
@@ -1297,6 +1312,7 @@ typedef enum {
 	_1gtk_1tree_1view_1get_1columns_FUNC,
 	_1gtk_1tree_1view_1get_1cursor_FUNC,
 	_1gtk_1tree_1view_1get_1expander_1column_FUNC,
+	_1gtk_1tree_1view_1get_1grid_1lines_FUNC,
 	_1gtk_1tree_1view_1get_1headers_1visible_FUNC,
 	_1gtk_1tree_1view_1get_1path_1at_1pos_FUNC,
 	_1gtk_1tree_1view_1get_1rules_1hint_FUNC,
@@ -1311,14 +1327,12 @@ typedef enum {
 	_1gtk_1tree_1view_1scroll_1to_1point_FUNC,
 	_1gtk_1tree_1view_1set_1cursor_FUNC,
 	_1gtk_1tree_1view_1set_1drag_1dest_1row_FUNC,
-	_1gtk_1tree_1view_1set_1enable_1search_FUNC,
 	_1gtk_1tree_1view_1set_1grid_1lines_FUNC,
 	_1gtk_1tree_1view_1set_1headers_1visible_FUNC,
 	_1gtk_1tree_1view_1set_1model_FUNC,
 	_1gtk_1tree_1view_1set_1rules_1hint_FUNC,
 	_1gtk_1tree_1view_1set_1search_1column_FUNC,
 	_1gtk_1tree_1view_1unset_1rows_1drag_1dest_FUNC,
-	_1gtk_1tree_1view_1widget_1to_1tree_1coords_FUNC,
 	_1gtk_1vbox_1new_FUNC,
 	_1gtk_1viewport_1set_1shadow_1type_FUNC,
 	_1gtk_1vscale_1new_FUNC,
@@ -1393,6 +1407,7 @@ typedef enum {
 	_1gtk_1widget_1set_1default_1direction_FUNC,
 	_1gtk_1widget_1set_1direction_FUNC,
 	_1gtk_1widget_1set_1double_1buffered_FUNC,
+	_1gtk_1widget_1set_1halign_FUNC,
 	_1gtk_1widget_1set_1has_1window_FUNC,
 	_1gtk_1widget_1set_1mapped_FUNC,
 	_1gtk_1widget_1set_1name_FUNC,
@@ -1403,6 +1418,7 @@ typedef enum {
 	_1gtk_1widget_1set_1size_1request_FUNC,
 	_1gtk_1widget_1set_1state_FUNC,
 	_1gtk_1widget_1set_1tooltip_1text_FUNC,
+	_1gtk_1widget_1set_1valign_FUNC,
 	_1gtk_1widget_1set_1visible_FUNC,
 	_1gtk_1widget_1shape_1combine_1region_FUNC,
 	_1gtk_1widget_1show_FUNC,
@@ -1650,11 +1666,6 @@ typedef enum {
 #else
 	memmove__JLorg_eclipse_swt_internal_gtk_PangoAttribute_2J_FUNC,
 #endif
-#ifndef JNI64
-	memmove__ILorg_eclipse_swt_internal_gtk_XClientMessageEvent_2I_FUNC,
-#else
-	memmove__JLorg_eclipse_swt_internal_gtk_XClientMessageEvent_2J_FUNC,
-#endif
 #ifndef JNI64
 	memmove__ILorg_eclipse_swt_internal_gtk_XExposeEvent_2I_FUNC,
 #else
@@ -1725,11 +1736,6 @@ typedef enum {
 #else
 	memmove__Lorg_eclipse_swt_internal_gtk_GdkEventScroll_2JJ_FUNC,
 #endif
-#ifndef JNI64
-	memmove__Lorg_eclipse_swt_internal_gtk_GdkEventVisibility_2II_FUNC,
-#else
-	memmove__Lorg_eclipse_swt_internal_gtk_GdkEventVisibility_2JJ_FUNC,
-#endif
 #ifndef JNI64
 	memmove__Lorg_eclipse_swt_internal_gtk_GdkEventWindowState_2II_FUNC,
 #else
@@ -1770,21 +1776,11 @@ typedef enum {
 #else
 	memmove__Lorg_eclipse_swt_internal_gtk_GtkCellRendererClass_2J_FUNC,
 #endif
-#ifndef JNI64
-	memmove__Lorg_eclipse_swt_internal_gtk_GtkColorSelectionDialog_2I_FUNC,
-#else
-	memmove__Lorg_eclipse_swt_internal_gtk_GtkColorSelectionDialog_2J_FUNC,
-#endif
 #ifndef JNI64
 	memmove__Lorg_eclipse_swt_internal_gtk_GtkFixed_2I_FUNC,
 #else
 	memmove__Lorg_eclipse_swt_internal_gtk_GtkFixed_2J_FUNC,
 #endif
-#ifndef JNI64
-	memmove__Lorg_eclipse_swt_internal_gtk_GtkSelectionData_2II_FUNC,
-#else
-	memmove__Lorg_eclipse_swt_internal_gtk_GtkSelectionData_2JJ_FUNC,
-#endif
 #ifndef JNI64
 	memmove__Lorg_eclipse_swt_internal_gtk_GtkWidgetClass_2I_FUNC,
 #else
@@ -1835,12 +1831,10 @@ typedef enum {
 #else
 	memmove__Lorg_eclipse_swt_internal_gtk_XFocusChangeEvent_2JJ_FUNC,
 #endif
-#ifndef JNI64
-	memmove__Lorg_eclipse_swt_internal_gtk_XVisibilityEvent_2II_FUNC,
-#else
-	memmove__Lorg_eclipse_swt_internal_gtk_XVisibilityEvent_2JJ_FUNC,
-#endif
+	pangoFontFaceNewProc_1CALLBACK_FUNC,
+	pangoFontFamilyNewProc_1CALLBACK_FUNC,
 	pangoLayoutNewProc_1CALLBACK_FUNC,
+	printerOptionWidgetNewProc_1CALLBACK_FUNC,
 	realpath_FUNC,
 	strcmp_FUNC,
 } OS_FUNCS;
